#!/bin/csh -f
# Script to run the program unresCSA on linux with MPICH (parallel version).
#
setenv env $HOME/.env
rm -f $env
touch $env

echo "This job is running on host `hostname` "
if ($#argv<2) then
  echo "You must supply potential type and input/output filename prefix."
  exit (1)
endif

set DD = $HOME/UNRES/PARAM
echo "setenv DD $HOME/UNRES/PARAM " > $env
set BIN = $HOME/UNRES/NEW/bin
echo "setenv BIN $HOME/UNRES/NEW/bin " > $env

set POT = $1
setenv POT $1
echo "setenv POT $1 " >> $env

setenv PREFIX $2
echo "setenv PREFIX $2 " >> $env

#
# Is the specified potential in the list of valid force fields?
#
if (($POT != "LJ") && ($POT != "LJK") && ($POT != "BP") && ($POT != "GB") \
&& ($POT != "GBV") ) then
  echo "Unknown potential type $POT"
  exit(1)
endif

# Check for the existence of the input file

if (!(-e $2.inp)) then
  echo "Error: The input file $2.inp does not exist."
  exit(1)
endif

# 5/17/97 parameter file assignment was changed from link to SETENV statements
#
setenv THETPAR $DD/thetaml.5parm
echo "setenv THETPAR $DD/thetaml.5parm " >> $env
setenv ROTPAR $DD/scgauss.parm
echo "setenv ROTPAR $DD/scgauss.parm " >> $env
setenv TORPAR $DD/torsion_ecepp.parm
echo "setenv TORPAR $DD/torsion_ecepp.parm " >> $env
setenv ELEPAR $DD/electr.parm
echo "setenv ELEPAR $DD/electr.parm " >> $env
setenv SIDEPAR $DD/scinter_$1.parm
echo "setenv SIDEPAR $DD/scinter_$1.parm  " >> $env
setenv FOURIER $DD/fourier_GAP.parm
echo "setenv FOURIER $DD/fourier_GAP.parm " >> $env


# Pattern database
setenv PATTERN $DD/patterns.cart
echo "setenv PATTERN $DD/patterns.cart " >> $env

# Set parallel environment, depending on job type (interactive or batch).

setenv MP_PROCS $3
setenv FGPROCS $4
echo "setenv FGPROCS $4 " >> $env

set DEBUG = 0

limit coredumpsize 0
mpirun -gdb   -machinefile ./h -np $3 $BIN/unres7CSApg
 rm -f $env
 touch $env

# Clear empty files
find . -size 0 -exec rm {} \;
