#define _LOOSE_KERNEL_NAMES
 
#include <stdio.h>
 
#define __SIGFPE_INTERNAL
#define NON_PORTABLE_FENV
#include "fenv_loc.h"
 
/* A place to keep the unmasked "sticky" bits. */
static int sticky;

 
int
__sigfpe_abort(char *s)
{
  char line[81];
  FILE *cmd;
    
  cmd = fopen("/proc/self/cmdline", "r");
  if ( cmd )
    fscanf(cmd, "%80s", line);
  else
    line[0] = 0;
  line[80] = 0;
  fprintf(stderr, "FATAL: %s %s at %p\n",
          line, s);
  exit(1);
  return 0;  /* Keep gcc happy. */
}
 
 
int default_stack_flt_handler(struct sigcontext_struct *sig,
			      struct i387_hard_struct *fpu)
{
  return __sigfpe_abort("got FPU stack fault");
}
 
 
int default_invalid_handler(struct sigcontext_struct *sig,
			      struct i387_hard_struct *fpu)
{
  return __sigfpe_abort("attempted invalid FPU operation");
}
 
 
int default_divbyzero_handler(struct sigcontext_struct *sig,
			      struct i387_hard_struct *fpu)
{
  return __sigfpe_abort("attempted FPU divide-by-zero");
}
 


struct fptraps __trap_vectors =
{ default_invalid_handler, default_stack_flt_handler, 0,
  default_divbyzero_handler, 0, 0, 0,
  0, 1
};

 
/*
  Enable exception traps for the exceptions specified by traps.
  */
int feenabletraps(int traps)
{
  unsigned short cw;
 
  asm volatile ("fnstcw %0":"=m" (cw));
  if ( ! (traps & ~(FE_ALL_EXCEPT | FE_STACK_FLT)) )
    {
      if ( traps & (FE_INVALID | FE_STACK_FLT) )
	{
	  if ( traps & FE_INVALID )
	    __trap_vectors.trap_invalid = 1;
	  if ( traps & FE_STACK_FLT )
	    __trap_vectors.trap_stack_flt = 1;
	  traps |= FE_INVALID;
	}
      cw &= ~traps;
      asm volatile ("fldcw %0"::"m" (cw));
      return 1;
    }
  else
    return 0;
}
 
 
/*
  Disable exception traps for the exceptions specified by traps.
  */
int fedisabletraps(int traps)
{
  unsigned short cw;
 
  asm volatile ("fnstcw %0":"=m" (cw));
  if ( ! (traps & ~(FE_ALL_EXCEPT | FE_STACK_FLT)) )
    {
      if ( traps & (FE_INVALID | FE_STACK_FLT) )
	{
	  if ( traps & FE_INVALID )
	    __trap_vectors.trap_invalid = 0;
	  if ( traps & FE_STACK_FLT )
	    __trap_vectors.trap_stack_flt = 0;
	  if ( !(__trap_vectors.trap_invalid | __trap_vectors.trap_stack_flt) )
	    traps |= FE_INVALID;
	}
      cw |= (traps & FE_ALL_EXCEPT);
      asm volatile ("fldcw %0"::"m" (cw));
      return 1;
    }
  else
    return 0;
}
 
 
/*
  Get the trap vector for the specified exception.
 */
int fegetvector(ftrap_t *vector, int excep)
{
  switch ( excep )
    {
    case FE_STACK_FLT:
      *vector = __trap_vectors.stack_flt;
      break;
    case FE_INVALID:
      *vector = __trap_vectors.invalid;
      break;
    case FE_DIVBYZERO:
      *vector = __trap_vectors.divbyzero;
      break;
    case FE_OVERFLOW:
      *vector = __trap_vectors.overflow;
      break;
    case FE_INEXACT:
      *vector = __trap_vectors.inexact;
      break;
    case FE_UNDERFLOW:
      *vector = __trap_vectors.underflow;
      break;
    case FE_DENORM_OP:
      *vector = __trap_vectors.denorm_op;
      break;
    default:
      return 0;
    }
  return 1;
}
 
 
/*
  Set the trap vector for the specified exception.
 */
int fesetvector(ftrap_t *vector, int excep)
{
  switch ( excep )
    {
    case FE_STACK_FLT:
      __trap_vectors.stack_flt = *vector;
      break;
    case FE_INVALID:
      __trap_vectors.invalid = *vector;
      break;
    case FE_DIVBYZERO:
      __trap_vectors.divbyzero = *vector;
      break;
    case FE_OVERFLOW:
      __trap_vectors.overflow = *vector;
      break;
    case FE_INEXACT:
      __trap_vectors.inexact = *vector;
      break;
    case FE_UNDERFLOW:
      __trap_vectors.underflow = *vector;
      break;
    case FE_DENORM_OP:
      __trap_vectors.denorm_op = *vector;
      break;
    default:
      return 0;
    }
  return 1;
}
 
#define FE_REASONABLE_EXCEPT    ( FE_INEXACT  | FE_DIVBYZERO                                               | FE_OVERFLOW  | FE_INVALID )
 
void fpenab_ () {
feenabletraps(FE_REASONABLE_EXCEPT);
  }
